<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Stripe\Service;

/**
 * Abstract base class for all service factories used to expose service
 * instances through {@link \Stripe\StripeClient}.
 *
 * Service factories serve two purposes:
 *
 * 1. Expose properties for all services through the `__get()` magic method.
 * 2. Lazily initialize each service instance the first time the property for
 *    a given service is used.
 */
abstract class AbstractServiceFactory {
	/** @var \Stripe\StripeClientInterface */
	private $client;

	/** @var array<string, AbstractService|AbstractServiceFactory> */
	private $services;

	/**
	 * @param \Stripe\StripeClientInterface $client
	 */
	public function __construct($client) {
		$this->client   = $client;
		$this->services = [];
	}

	/**
	 * @param string $name
	 *
	 * @return null|string
	 */
	abstract protected function getServiceClass($name);

	/**
	 * @param string $name
	 *
	 * @return null|AbstractService|AbstractServiceFactory
	 */
	public function __get($name) {
		return $this->getService($name);
	}

	/**
	 * @param string $name
	 *
	 * @return null|AbstractService|AbstractServiceFactory
	 */
	public function getService($name) {
		$serviceClass = $this->getServiceClass($name);

		if (null !== $serviceClass) {
			if (! \array_key_exists($name, $this->services)) {
				$this->services[$name] = new $serviceClass($this->client);
			}

			return $this->services[$name];
		}

		\trigger_error('Undefined property: ' . static::class . '::$' . $name);

		return null;
	}
}
